
#include <stdio.h>
#include <windows.h>
#include "..\include\lzss_compress.h"
#include "..\include\lzss_uncompress.h"

int main(int argc, char* argv[])
{
	HS_LZSS_Compress	oLZSS_Compress;			// Our compression class
	HS_LZSS_Uncompress	oLZSS_Uncompress;		// Our Uncompression class
	FILE			*fptr;

	unsigned char	*bmyData;
	unsigned char	*bmyCompressedData;

	unsigned long	nCompressedSize;
	unsigned long	nUncompressedSize;
	int				nRes;

	// Need 3 command line parameters
	if (argc != 4)
	{
		printf("Usage: LZSS <-c|-u> <infile> <outfile>\n");
		return 0;
	}

	// Compress function
	if (!stricmp("-c", argv[1]))
	{
		// Open our input file
		if ( (fptr = fopen(argv[2], "rb")) == NULL)
		{
			printf("Error opening input file.\n");
			return 0;	// Error
		}

		// How big is the file?
		fseek(fptr, 0, SEEK_END);
		nUncompressedSize = ftell(fptr);
		printf("Input file size      : %d\n", nUncompressedSize);

		// Create our memory buffers as the same size
		bmyData = (unsigned char *)malloc(nUncompressedSize);
		bmyCompressedData = (unsigned char *)malloc(nUncompressedSize);
		
		// Read in the entire input file then close it
		fseek(fptr, 0, SEEK_SET);
		fread(bmyData, sizeof(unsigned char), nUncompressedSize, fptr);
		fclose(fptr);

		// Do the compression
		nRes = oLZSS_Compress.Compress(bmyData, bmyCompressedData, nUncompressedSize, &nCompressedSize, 2);
		if (nRes != HS_LZSS_E_OK)
		{
			printf("File not worth compressing.\n");
			return 0;
		}

		// Write out the data
		if ( (fptr = fopen(argv[3], "w+b")) == NULL)
		{
			printf("Error opening output file.\n");
			free(bmyData);
			free(bmyCompressedData);
			return 0;	// Error
		}

		// Write out the data
		fwrite(bmyCompressedData, sizeof(unsigned char), nCompressedSize, fptr);
		fclose(fptr);

		// Print the output size
		printf("Output file size     : %d\n", nCompressedSize);
		printf("Compression ratio    : %.2f%%\n", 
			100 - (((float)nCompressedSize / (float)nUncompressedSize)*100.0) );

		// Free our memory buffers
		free(bmyData);
		free(bmyCompressedData);

		return 0;
	}





	// Uncompress function
	if (!stricmp("-u", argv[1]))
	{
		// Open our input file
		if ( (fptr = fopen(argv[2], "rb")) == NULL)
		{
			printf("Error opening input file.\n");
			return 0;	// Error
		}

		// How big is the file?
		fseek(fptr, 0, SEEK_END);
		nCompressedSize = ftell(fptr);
		printf("Input file size      : %d\n", nCompressedSize);
 
		// Create our memory buffer for the compressed data
		bmyCompressedData = (unsigned char *)malloc(nCompressedSize);
		
		// Read in the entire input file then close it
		fseek(fptr, 0, SEEK_SET);
		fread(bmyCompressedData, sizeof(unsigned char), nCompressedSize, fptr);
		fclose(fptr);

		// Work out the size of the uncompressed data and allocate
		nRes = oLZSS_Uncompress.GetUncompressedSize(bmyCompressedData, &nUncompressedSize);
		if (nRes != HS_LZSS_E_OK)
		{
			printf("Error not a valid LZSS file.\n");
			return 0;
		}

		// Allocate memory buffer for uncompressed data
		bmyData = (unsigned char *)malloc(nUncompressedSize);


		// Do the uncompression
		oLZSS_Uncompress.Uncompress(bmyCompressedData, bmyData);


		// Write out the data
		if ( (fptr = fopen(argv[3], "w+b")) == NULL)
		{
			printf("Error opening output file.\n");
			free(bmyData);
			free(bmyCompressedData);
			return 0;	// Error
		}

		// Write out the data
		fwrite(bmyData, sizeof(unsigned char), nUncompressedSize, fptr);
		fclose(fptr);

		// Print filesize
		printf("Output file size     : %d\n", nUncompressedSize);

		// Free our memory buffers
		free(bmyData);
		free(bmyCompressedData);

		return 0;

	}
	
	return 0;
}
